package pay.platform.api.servcie.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.RandomUtil;
import com.alipay.api.AlipayClient;
import com.alipay.api.domain.AlipayTradePrecreateModel;
import com.alipay.api.response.AlipayTradePrecreateResponse;
import org.apache.commons.lang3.StringUtils;
import org.jetbrains.annotations.NotNull;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import pay.platform.alipay.enums.AliPayApi;
import pay.platform.alipay.enums.AliPayApiConfig;
import pay.platform.api.model.request.PayRequest;
import pay.platform.api.model.request.PayTestRequest;
import pay.platform.api.model.response.PayResponse;
import pay.platform.api.servcie.*;
import pay.platform.core.config.PayConfig;
import pay.platform.core.exception.BusinessException;
import pay.platform.core.utils.QRCodeUtil;
import pay.platform.core.utils.SignUtil;
import pay.platform.domain.*;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Calendar;
import java.util.List;
import java.util.Locale;
import java.util.TreeMap;

/**
 *
 **/
@Service
public class PayServiceImpl implements PayService {

    private static final Logger log = LoggerFactory.getLogger(PayServiceImpl.class);
    @Autowired
    private PayMerchantAppKeyService payMerchantAppKeyService;
    @Autowired
    private PayOrderService payOrderService;
    @Autowired
    private PayChannelService payChannelService;
    @Autowired
    private PayAgentService payAgentService;
    @Autowired
    private PayMerchantService payMerchantService;
    @Autowired
    private PayPlatformSettingService payPlatformSettingService;
    @Autowired
    private PayConfig payConfig;
    @Autowired
    private PayDailyTotalService payDailyTotalService;


    @Override
    @Transactional
    public PayResponse pay(PayRequest payRequest) {
        var result = getResult(payRequest);

        //验证签名
        validSign(payRequest, result.payMerchantAppKey().getAppKey());
        var orderInfo = payOrderService.queryByOrderNo(result.payMerchantAppKey().getMerchantId(), payRequest.getOrderNo());
        if (orderInfo != null) {
            throw new BusinessException("订单号已存在");
        }
        //通过代理获取支付通道-》聚合支付-》店铺
        PayChannel payChannel = randomChannel(result.agentId());
        return getPayResponse(result, payChannel, createOrder(payRequest, result.payMerchantAppKey(), payChannel, result.payAgent(), result.payMerchant()));
    }

    @Override
    @Transactional
    public PayResponse payTest(PayTestRequest payRequest) {
        var result = getResult(payRequest);

        //验证签名
        validSign(payRequest, result.payMerchantAppKey().getAppKey());
        var orderInfo = payOrderService.queryByOrderNo(result.payMerchantAppKey().getMerchantId(), payRequest.getOrderNo());
        if (orderInfo != null) {
            throw new BusinessException("订单号已存在");
        }
        PayChannel payChannel = null;
        //指定通道
        if (payRequest.getChannelId() != null) {
            payChannel = payChannelService.getById(payRequest.getChannelId());
        } else {
            //通过代理获取支付通道-》聚合支付-》店铺
            payChannel = randomChannel(result.agentId());
        }


        return getPayResponse(result, payChannel, createOrder(payRequest, result.payMerchantAppKey(), payChannel, result.payAgent(), result.payMerchant()));

    }

    private PayResponse getPayResponse(Result result, PayChannel payChannel, PayOrder payRequest) {
        if (payRequest.getAmount().compareTo(payChannel.getOnePayMaxAmount()) > 0) {
            throw new BusinessException(STR."单笔支付金额最高：\{payChannel.getOnePayMaxAmount()}");
        }
        var payDailyTotal = payDailyTotalService.totalDailyPayAmount(payChannel.getId());
        if (payDailyTotal != null) {
            if (payDailyTotal.getAmount().add(payRequest.getAmount()).compareTo(payChannel.getDayMaxPayAmount()) > 0) {
                throw new BusinessException("当天支付金额已上限");
            }
        }
        var list = payPlatformSettingService.list();
        if (CollectionUtil.isEmpty(list)) {
            throw new BusinessException("平台配置为空");
        }
        var payPlatformSetting = list.get(0);
//        //创建本地订单
        PayOrder payOrder = payRequest;
        //普通商户模式
        if (payChannel.getType() != null && payChannel.getType() == 2) {
            payPlatformSetting.setAppId(payChannel.getAppId());
            payPlatformSetting.setPrivateKey(payChannel.getPrivateKey());
            payPlatformSetting.setPublicKey(payChannel.getPublicKey());
        }
//        //创建支付订单
        AlipayTradePrecreateResponse alipayQrcodeResponse = createPayOrder(payChannel, payOrder, payPlatformSetting, result.payMerchantAppKey().getAppId());
        return PayResponse.builder().qrCode(alipayQrcodeResponse.getQrCode()).imageBase64("data:image/png;base64,".concat(QRCodeUtil.qrCodeBase64(alipayQrcodeResponse.getQrCode()))).build();
    }

    private @NotNull Result getResult(PayRequest payRequest) {
        PayMerchantAppKey payMerchantAppKey = payMerchantAppKeyService.getByAppId(payRequest.getAppId());
        if (payMerchantAppKey == null) {
            throw new BusinessException("appId不存在");
        }
        PayMerchant payMerchant = payMerchantService.getByMerchantId(payMerchantAppKey.getMerchantId());
        if (payMerchant == null) {
            throw new BusinessException("商户不存在");
        }
        if (payMerchant.getStatus() == 0) {
            throw new BusinessException("商户已禁用");
        }
        PayAgent payAgent = payAgentService.getByAgentId(payMerchantAppKey.getAgentId());
        String agentId = payMerchantAppKey.getAgentId();
        if (StringUtils.isBlank(agentId)) {
            throw new BusinessException("代理商不能为空");
        }
        if (payAgent == null) {
            throw new BusinessException("代理不存在");
        }
        if (payAgent.getStatus() == 0) {
            throw new BusinessException("代理已禁用");
        }
        Result result = new Result(payMerchantAppKey, payMerchant, payAgent, agentId);
        return result;
    }

    private record Result(PayMerchantAppKey payMerchantAppKey, PayMerchant payMerchant, PayAgent payAgent,
                          String agentId) {
    }

    private void validSign(PayRequest payRequest, String appKey) {
        TreeMap<String, String> treeMap = new TreeMap<>();
        treeMap.put("appId", payRequest.getAppId());
        treeMap.put("body", payRequest.getBody());
        treeMap.put("subject", payRequest.getSubject());
        treeMap.put("notifyUrl", payRequest.getNotifyUrl());
        treeMap.put("amount", payRequest.getAmount().toString());
        treeMap.put("orderNo", payRequest.getOrderNo());
        treeMap.put("currentTime", payRequest.getCurrentTime().toString());
        // 校验商户
        String sign = SignUtil.getSign(appKey, treeMap);
        if (!StringUtils.equals(payRequest.getSign().toLowerCase(Locale.ROOT), sign)) {
            throw new BusinessException("签名错误");
        }
    }

    public AlipayTradePrecreateResponse createPayOrder(PayChannel payChannel, PayOrder payOrder, PayPlatformSetting platformSetting, String merchantAppId) {

        //signType=rsa时，可不传 alipay_public_key
        //alipayCore = clientBuilder.setAppId(appid).setPrivateKey(private_key).setSignType(AlipayConstants.SIGN_TYPE_RSA).build();

        //signType=rsa2时，必须传 支付宝公钥 alipay_public_key
        AliPayApiConfig aliPayApiConfig = AliPayApiConfig.builder()
                .setAppId(platformSetting.getAppId())
                .setAliPayPublicKey(platformSetting.getPublicKey())
                .setPrivateKey(platformSetting.getPrivateKey())
                .setServiceUrl(AliPayApiConfig.DEFAULT_GATEWAY_URL)
                .build();
        AlipayTradePrecreateModel model = new AlipayTradePrecreateModel();
        if (payChannel.getType() != null && payChannel.getType() == 1) {
            model.setSellerId(payChannel.getSellerId());
        }
        model.setSubject(payOrder.getSubject());
        model.setOutTradeNo(payOrder.getPlatformOrderNo());
        model.setBody(payOrder.getRemark());
        model.setTotalAmount(payOrder.getAmount().toString());

        AlipayClient aliPayClient = aliPayApiConfig.getAliPayClient();

        AlipayTradePrecreateResponse response = null;
        try {
            response = AliPayApi.tradePrecreatePayToResponse(aliPayClient, model, payConfig.getCallbackUrl().concat(merchantAppId), null);
        } catch (Exception e) {
            //设置通道不可用
            payChannel.setStatus(0);
            payChannel.setErrorMessage(e.getMessage());
            payChannelService.updateInfo(payChannel);
            throw new BusinessException(e.getMessage());
        }

        if (!response.isSuccess()) {
            //设置通道不可用
            payChannel.setStatus(0);
            payChannel.setErrorMessage(response.getSubMsg());
            payChannelService.updateInfo(payChannel);
            throw new BusinessException(response.getSubMsg());
        }
        return response;
    }

    /**
     * 创建本地订单
     *
     * @param payRequest
     * @param payMerchantAppKey
     * @return
     */
    private PayOrder createOrder(PayRequest payRequest, PayMerchantAppKey payMerchantAppKey, PayChannel payChannel, PayAgent payAgent, PayMerchant payMerchant) {
        Calendar calendar = Calendar.getInstance();
        PayOrder payOrder = PayOrder.builder()
                .amount(payRequest.getAmount())
                .subject(payRequest.getSubject())
                .remark(payRequest.getBody())
                .notifyUrl(payRequest.getNotifyUrl())
                .platformOrderNo(STR."\{System.currentTimeMillis()}".concat(RandomUtil.randomNumbers(8)))
                .agentId(payMerchantAppKey.getAgentId())
                .status(0)
                .year(calendar.get(Calendar.YEAR))
                .month(calendar.get(Calendar.MONTH) + 1)
                .day(calendar.get(Calendar.DAY_OF_YEAR))
                .week(calendar.get(Calendar.WEEK_OF_YEAR))
                .channelId(payChannel.getId())
                .channelName(payChannel.getName())
                .rate(payChannel.getRate())
                .orderNo(payRequest.getOrderNo())
                .merchantId(payMerchantAppKey.getMerchantId())
                .build();

            //商户费率
            if (payMerchant != null && payMerchant.getRate() != null && payMerchant.getRate().compareTo(BigDecimal.ZERO) > 0) {
                var agentRateAmount = payRequest.getAmount().multiply(payMerchant.getRate()).divide(BigDecimal.valueOf(1000)).setScale(2, RoundingMode.HALF_UP);
                payOrder.setAgentRate(payMerchant.getRate());
                payOrder.setAgentRateAmount(agentRateAmount);
            }
            //代理费率
            if (payAgent != null && payAgent.getRate() != null && payAgent.getRate().compareTo(BigDecimal.ZERO) > 0) {
                var agentRateAmount = payRequest.getAmount().multiply(payAgent.getRate()).divide(BigDecimal.valueOf(1000)).setScale(2, RoundingMode.HALF_UP);
                payOrder.setPlatformRate(payAgent.getRate());
                payOrder.setPlatformRateAmount(agentRateAmount);
            }
        payOrderService.save(payOrder);
        return payOrder;


    }


    /**
     * 随机通道
     *
     * @param agentId
     * @return
     */
    private PayChannel randomChannel(String agentId) {
        List<PayChannel> payChannels = payChannelService.findChannel(agentId);
        if (CollectionUtil.isEmpty(payChannels)) {
            throw new BusinessException("无可用通道");
        }
        return payChannels.get(RandomUtil.randomInt(payChannels.size()));
    }

    private PayMerchantAppKey validateMerchant(PayMerchantAppKey payMerchantAppKey, PayMerchant payMerchant, PayAgent payAgent) {
        if (payMerchantAppKey == null) {
            throw new BusinessException("appId不存在");
        }

        if (payMerchant == null) {
            throw new BusinessException("商户不存在");
        }
        if (payMerchant.getStatus() == 0) {
            throw new BusinessException("商户已禁用");
        }
        String agentId = payMerchantAppKey.getAgentId();
        if (StringUtils.isBlank(agentId)) {
            throw new BusinessException("代理商不能为空");
        }

        if (payAgent == null) {
            throw new BusinessException("代理不存在");
        }
        if (payAgent.getStatus() == 0) {
            throw new BusinessException("代理已禁用");
        }
        return payMerchantAppKey;
    }
}
